#include <vortex-python/common.hpp>

#if defined(VORTEX_ENABLE_ALAZAR)
#  include <vortex/driver/alazar/board.hpp>
#endif

void bind_driver_alazar(py::module& root) {
#if defined(VORTEX_ENABLE_ALAZAR)

    auto m = root.def_submodule("alazar");

    py::enum_<vortex::alazar::channel_t>(m, "Channel")
        .value("A", vortex::alazar::channel_t::A)
        .value("B", vortex::alazar::channel_t::B)
        .value("C", vortex::alazar::channel_t::C)
        .value("D", vortex::alazar::channel_t::D)
        .value("E", vortex::alazar::channel_t::E)
        .value("F", vortex::alazar::channel_t::F)
        .value("G", vortex::alazar::channel_t::G)
        .value("H", vortex::alazar::channel_t::H)
        .value("I", vortex::alazar::channel_t::I)
        .value("J", vortex::alazar::channel_t::J)
        .value("K", vortex::alazar::channel_t::K)
        .value("L", vortex::alazar::channel_t::L)
        .value("M", vortex::alazar::channel_t::M)
        .value("N", vortex::alazar::channel_t::N)
        .value("O", vortex::alazar::channel_t::O)
        .value("P", vortex::alazar::channel_t::P);

    py::enum_<vortex::alazar::coupling_t>(m, "Coupling")
        .value("DC", vortex::alazar::coupling_t::DC)
        .value("AC", vortex::alazar::coupling_t::AC);

    py::enum_<vortex::alazar::clock_edge_t>(m, "ClockEdge")
        .value("Rising", vortex::alazar::clock_edge_t::rising)
        .value("Falling", vortex::alazar::clock_edge_t::falling);

    py::enum_<vortex::alazar::trigger_slope_t>(m, "TriggerSlope")
        .value("Positive", vortex::alazar::trigger_slope_t::positive)
        .value("Negative", vortex::alazar::trigger_slope_t::negative);

    m.attr("InifinteAcquisition") = vortex::alazar::infinite_acquisition;
    m.attr("TriggerRangeTTL") = vortex::alazar::trigger_range_TTL;


    {
        using C = vortex::alazar::board_t::info_t::type_t;
        auto c = py::class_<C>(m, "InfoType");

        RW_VAR(id);
        RW_VAR(model);

        SHALLOW_COPY();
    }
    {
        using C = vortex::alazar::board_t::info_t::pcie_t;
        auto c = py::class_<C>(m, "InfoPCIe");

        RW_VAR(speed);
        RW_VAR(width);
        RW_VAR(speed_gbps);

        SHALLOW_COPY();
    }
    {
        using C = vortex::alazar::board_t::info_t::input_combination_t;
        auto c = py::class_<C>(m, "InfoInputCombination");

        RW_VAR(impedance_ohms);
        RW_VAR(input_range_millivolts);

        SHALLOW_COPY();
    }
    {
        using C = vortex::alazar::board_t::info_t;
        auto c = py::class_<C>(m, "Info");

        RW_VAR(system_index);
        RW_VAR(board_index);
        RW_VAR(serial_number);
        RW_VAR(calibration_date);
        RW_VAR(onboard_memory_size);
        RW_VAR(bits_per_sample);
        RW_VAR(type);
        RW_VAR(pcie);

        RW_VAR(supported_sampling_rates);
        RO_ACC(max_sampling_rate);

        RW_VAR(supported_input_combinations);

        RW_VAR(max_pretrigger_samples);
        RW_VAR(min_samples_per_record);

        RW_VAR(sample_alignment_divisor);
        FXN(nearest_aligned_samples_per_record);
        FXN(smallest_aligned_samples_per_record);

        RW_VAR(dual_port_memory);
        RW_VAR(dual_edge_sampling);

        SHALLOW_COPY();
    }

    {
        using C = vortex::alazar::board_t;
        auto c = py::class_<C, std::shared_ptr<C>>(m, "Board");

        c.def(py::init());
        c.def(py::init<U32, U32>());

        RO_ACC(info);

        // TODO: supply other methods
    }

    m.def("enumerate", vortex::alazar::enumerate);

#endif
}
