#pragma once

#include <functional>

#include <vortex/core.hpp>

namespace vortex::engine {

    struct timing_t {
        vortex::time_t create, service, scan, acquire, process, format, recycle;
    };

    enum class event_t {
        launch,     // session is starting up
        start,      // all acquisitions have started
        run,        // session has spawned all child threads
        stop,       // session is waiting for all inflight tasks to finish
        complete,   // scan has completed or all blocks have been dispatched
        shutdown,   // scan has ended early (i.e., no completed)
        exit,       // session has exited all child threads
        error,      // session has encountered an error
        abort       // session has encountered an error and cannot continue
    };
    using event_callback_t = std::function<void(event_t, std::exception_ptr)>;

    struct session_status_t {
        size_t allocated, inflight, dispatched, limit;

        auto available() const {
            return allocated - inflight;
        }

        auto progress() const {
            if (limit > 0) {
                return dispatched / double(limit);
            } else {
                return 0.0;
            }
        }

        auto utilization() const {
            return available() / double(allocated);
        }
    };

    using job_callback_t = std::function<void(size_t, session_status_t, timing_t)>;

}
